
#include <zephyr/kernel.h>
#include <zephyr/sys/reboot.h>
#include <inttypes.h>
#include <zephyr/logging/log.h>
#include <SEGGER_RTT.h>
#include "user_sensor_info.h"

#include <zephyr/fs/fs.h>
#include <zephyr/fs/littlefs.h>
#include <zephyr/storage/flash_map.h>

LOG_MODULE_REGISTER(SEGGER_INPUT, LOG_LEVEL_INF);

static sensor_info_t *sensor_info_p;

#define LFS_MOUNT_POINT "/lfs"
#define READ_SIZE 16
#define PROG_SIZE 16
#define CACHE_SIZE 16
#define LOOKAHEAD_SIZE 16
#define BLOCK_SIZE 4096
#define BLOCK_CYCLE 500

static uint8_t read_buffer[CACHE_SIZE];
static uint8_t prog_buffer[CACHE_SIZE];
static uint32_t lookahead_buffer[LOOKAHEAD_SIZE / sizeof(uint32_t)];

static struct fs_littlefs lfs_config = {
    .cfg = {
        .read_size = READ_SIZE,
        .prog_size = PROG_SIZE,
        .block_size = BLOCK_SIZE,
        .cache_size = CACHE_SIZE,
        .lookahead_size = LOOKAHEAD_SIZE,
        .block_cycles = BLOCK_CYCLE,
        .block_count = 5,
        .read_buffer = read_buffer,
        .prog_buffer = prog_buffer,
        .lookahead_buffer = lookahead_buffer,
        .file_max = 4096,
        .attr_max = 128,
    },
    .read_buffer = read_buffer,
    .prog_buffer = prog_buffer,
    .lookahead_buffer = lookahead_buffer,
};

static struct fs_mount_t my_lfs_mount = {
    .type = FS_LITTLEFS,
    .mnt_point = LFS_MOUNT_POINT,
    .storage_dev = (void *)FIXED_PARTITION_ID(littlefs_storage),
    .fs_data = &lfs_config,
};

void littlefs_clear_all(void)
{
    int ret;

    fs_mount(&my_lfs_mount);
    ret = fs_unmount(&my_lfs_mount);
    if (ret < 0)
    {
        LOG_ERR("Failed to unmount LittleFS: %d\n", ret);
        return;
    }
    ret = lfs_format(&lfs_config.lfs, &lfs_config.cfg);
    if (ret < 0)
    {
        LOG_ERR("Failed to format LittleFS: %d\n", ret);
        return;
    }

    LOG_INF("LittleFS formatted successfully. All files cleared.\n");
    ret = fs_mount(&my_lfs_mount);
    if (ret < 0)
    {
        LOG_ERR("Failed to mount LittleFS after formatting: %d\n", ret);
    }
    else
    {
        LOG_INF("LittleFS mounted successfully after formatting.\n");
    }
}

void littlefs_init(void)
{
    int rc;
    my_lfs_mount.fs_data = &lfs_config;
    rc = fs_mount(&my_lfs_mount);
    if (rc < 0)
    {
        LOG_WRN("Failed to mount LittleFS at %s: %d.\n", my_lfs_mount.mnt_point, rc);
    }
    else
    {
        LOG_WRN("LittleFS mounted successfully at %s\n", LFS_MOUNT_POINT);
    }
}

void littlefs_test(void)
{
    struct fs_file_t file;
    fs_file_t_init(&file);

    if (fs_open(&file, LFS_MOUNT_POINT "/myfile.txt", FS_O_CREATE | FS_O_WRITE | FS_O_APPEND) == 0)
    {
        const char *data = "X";
        fs_write(&file, data, strlen(data));
        fs_close(&file);
        LOG_WRN("Data written to file\n");
    }
    else
    {
        LOG_WRN("Failed to open file for writing\n");
    }

    if (fs_open(&file, LFS_MOUNT_POINT "/myfile.txt", FS_O_READ) == 0)
    {
        char buffer[320];
        ssize_t bytes_read = fs_read(&file, buffer, sizeof(buffer) - 1);
        if (bytes_read >= 0)
        {
            buffer[bytes_read] = '\0';
            LOG_WRN("Read from file: %s\n", buffer);
        }
        fs_close(&file);
    }
    else
    {
        LOG_WRN("Failed to open file for reading\n");
    }
}

void user_task_segger_input(void *p1, void *p2, void *p3)
{
    uint8_t input_char;
    while (1)
    {
        sensor_info_p = (sensor_info_t *)p1;
        k_msleep(100);

        if (SEGGER_RTT_Read(0, &input_char, sizeof(input_char)) == 1)
        {
            LOG_INF("%c", input_char);
            switch (input_char)
            {
            case 'o':
            {
            }
            break;

            case 'c':
            {
            }
            break;

            case 'r':
            {
            }
            break;

            case 'i':
            {
                littlefs_init();
            }
            break;

            case 't':
            {
                littlefs_test();
            }
            break;

            case 'f':
            {
                littlefs_clear_all();
            }
            break;

            case 'R':
            {
                sys_reboot(SYS_REBOOT_COLD);
            }
            break;

            default:
                break;
            }
        }
    }
}
